/*

Program: Timer2_With_Interrupt.c
Purpose: Usage of Timer2 on PIC18F4550 to produce delays.
Compiler: MPLAB C18 v.3.22 Student Edition in MPLAB IDE v.8.15a.

Author: Predrag Pei (eng. Peshich), Belgrade, Serbia.
Board: EasyPIC3 by Mikroelektronika.
January 2009.

How the program (suppose) is working: Eanbling the Timer2 and assuming that EasyPIC3 is clocked by 8MHz osc. the source for Timer2 is Fosc/4 which means that the incoming clock stream is divided by 4 which produces 2MHz clock stream. This stream enters in prescaler which divides it by 16. This produces 125kHz clock stream which enters in comparator.
The PR2 register is set to 0x7C (124 dec.) which means that comparator will produce one pulse on every 125 pulses from timer TMR2. The result is that that we have 1kHz outcoming stream of pulses from comparator to postscaler which is configured to 1:1, just to proceed stream to TMR2IF (interrupt flag).
By this way we have 1mS pulse on TMR2IF!
Look the picture "Interrupt on PIC18F4550 by Timer2.wmf" in the project directory to figure out what do we want.
On every occurence of interrupt from Timer2 we increment counter which we can use for timing base for delays or for other purposes.

*/

#include "p18f4550.h"
#include "Config_Bits.h"

void Timer2(void);

#pragma code Low_Vector_Section=0x18		// We shall use low-interrupt. Interrupt vector is at address 0x18.
void Timer2_Interrupt (void)						// On every occurence of interrupt from Timer2...
{
	_asm
	goto	Timer2													// ... jump to low-interrupt ISR Timer2!
	_endasm
}
#pragma code

volatile unsigned int Limit=0, Counter=0;					// Global variables. Their values can be changed by main() or by ISR. 

#pragma interruptlow Timer2							// Tell the compiler that ISR for low-interrupt is Timer2.
void Timer2(void)												// The body of low-interrupt ISR starts here. 
{
	PIR1bits.TMR2IF=0;										// Immediately clear the interrupt flag.			
	Counter++;														// Increase counter. Remember, this means that one mS is passed!
	LATDbits.LATD0=!LATDbits.LATD0;				// Change the state of port D, pin 0 and of LED connected to it.
	if(Counter>=1000)											// Look if counter reaches 1000mS - one second (1s)! And if so...
	{
		LATDbits.LATD4=!LATDbits.LATD4;			// ... change state of port D, pin 4 and of LED connected to it...
		Counter=0;													// ... and clear counter to start counting again.
		Limit++;														// Increase limit.
	}		
}	

void main(void)
{
	TRISD=0x00;		 						// Port D to be output port.
	LATD=0x00;	
	IPR1bits.TMR2IP=0;				// Interrupt priority bit set to 0 - low priority.
	PIE1bits.TMR2IE=1;				// Interrupt on TMR2IF enabled.
	
	//T2CONbits.T2CKPS0=1;
	//T2CONbits.T2CKPS1=1;
	//T2CONbits.TMR2ON=1;
	 
	T2CON=0b00000110;					// Timer2 config register:	prescaler (bits 0-1):			divide by 16
	 													//													timer2 ON/OFF (bit 2):		TMR2ON=1 - ON
	 													// 													postscaler (bits 3-7):		1:1
	PR2=0x7C;									// Period register set to 124 (dec.).
	INTCONbits.PEIE=1;				// Enable peripheral interrupt sources.
	INTCONbits.GIE=1;					// Enable global interrupts.
	while(Limit<=5);
} 
